#!/bin/sh

# Implement rudimentary locking around the test suite when building
# under beehive to stop failures when two architectures are built on
# the same machine (like, for instance, ppc and ppc64).

# We need a directory that is shared between buildroots, and the only
# one that fits the bill is /mnt/redhat.  I feel really dirty using
# it, but I can't see another way around this.  I'll be taking a
# shower after I've finished writing this script.

[ -d /mnt/redhat/beehive ] || exit 0
lockbase=/mnt/redhat/.mod_perl-testlock
[ -d $lockbase ] || mkdir $lockbase 2>/dev/null || exit 0
lockdir=$lockbase/`hostname`

case $1 in
acquire) ;;
release) rm -Rf $lockdir ; echo "released lock" ; exit 0 ;;
*) echo >&2 "usage: $0 acquire|release"; exit 1
esac

sleeptime=0
while true; do
    if ! mkdir $lockdir 2>/dev/null; then
	pid=`cat $lockdir/pid`
	cmd1=`cat $lockdir/cmd`
	cmd2=`cat /proc/$pid/cmdline 2>/dev/null | tr '\0' ' ' | sed 's: $::'`
	if [ "$cmd1" = "$cmd2" ]; then
	    sleep 1
	    sleeptime=`expr $sleeptime + 1`
	    continue
	fi
    fi
    if [ ! -d $lockdir ]; then
        echo >&2 "error: unable to obtain lock"
	exit 1
    fi

    pid=`cat /proc/$$/stat | sed "s:^$$ ([^)]*) . \([0-9]\{1,\}\).*$:\1:"`
    cmd=`tr '\0' ' ' < /proc/$pid/cmdline | sed 's: $::'`
    echo $pid > $lockdir/pid
    echo $cmd > $lockdir/cmd
    echo "acquired lock (waited ${sleeptime}s)"
    exit 0
done
